<?php
/**
 * @package  Feedo
 */
/**
Plugin Name: Feedo
Plugin URI: http://regalix.com
Description: Feedo Plugin for Wordpress WooCommerce products.
Version: 1.2
Author: Regalix
Author URI: https://regalix.com
License: GPLv2 or later
Text Domain: feedo
*/

/**
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Copyright 20018-2033 Regalix Inc.
*/

/**
 * If this file is called directly, abort!!!
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Require once the Composer Autoload
 */
if ( file_exists( dirname( __FILE__ ) . '/vendor/autoload.php' ) ) {
	require_once dirname( __FILE__ ) . '/vendor/autoload.php';
}

/**
 * All required constants
 */
define( 'FEEDO_PLUGIN_PATH', plugin_dir_path( __FILE__ ));
define( 'FEEDO_PLUGIN_URL', plugin_dir_url( __FILE__ ));
define( 'FEEDO_PLUGIN',  plugin_basename( __FILE__ ) );
define( 'FEEDO_SERVER_URL', 'https://feedoapi.regalix.com/'); // FEEDO Server for optimizing
define( 'FEEDO_PLUGINURL', get_admin_url().'admin.php?page=feedo');
define( 'FEEDO_SETTINGURL', 'admin.php?page=feedo_settings');

/**
 * Add our API server (FEEDO Server): feedoapi.regalix.com to allowed_http_origins filter
 * For fixing Cross domain origin issue
 */
function feedo_add_cors_header( $origins ) {
    $origins[] = 'https://feedoapi.regalix.com'; // We call FEEDO server to optimize feed and save status.
    return $origins;
}
add_filter( 'allowed_http_origins', 'feedo_add_cors_header' );


/**
 * Activate plugin action
 */
function feedo_activate_plugin() {
	Inc\Base\Activate::feedoactivate();
}
register_activation_hook( __FILE__, 'feedo_activate_plugin' );


/**
 * Deactivate plugin action
 */
function feedo_deactivate_plugin() {
	Inc\Base\Deactivate::feedodeactivate();
}
register_deactivation_hook( __FILE__, 'feedo_deactivate_plugin' );


function feedo_uninstall_plugin() {
	$shopid = get_option('feedo_shopid');
	$ajaxdata = Inc\Common::getServerData('api/deletestore?shopid='.$shopid);
	delete_option( 'feedo_shopid');
	$tempdata = array(
		'shopid' => $shopid,
		'status'=>'Uninstall hook called',
		'step'=> 'UNINSTALL',
		'date'=> ''
	);
	$tempapi = Inc\Common::getServerData('api/update_track', $tempdata);
}
register_uninstall_hook(__FILE__, 'feedo_uninstall_plugin');

/**
 * Hooking Bulk product delete action to update FEEDO server
 */
function feedo_bulk_product_delete( $new_status, $old_status, $post ) {
	if( $post->post_type == 'product' && $new_status == 'trash'){
		$shopid = get_option('feedo_shopid');
		$data = array('shopid'=>$shopid, 'productid'=>$post->ID);
		$deleteInfo = Inc\Common::getServerData('api/delete_single_product', $data);
	}
}
add_action( 'transition_post_status', 'feedo_bulk_product_delete', 10, 3 );


/**
 * Hooking single product delete action to update FEEDO server
 */
function feedo_single_product_delete(){
    global $post;
    if('product' == $post->post_type) {
        $shopid = get_option('feedo_shopid');
		$data = array('shopid'=>$shopid, 'productid'=>$post->ID);
		$deleteInfo = Inc\Common::getServerData('api/delete_single_product', $data);
    }
}
add_action('publish_to_trash', 'feedo_single_product_delete');


/**
 * Uploads products from woocommerce to FEEDO server for the first time and once.
 * Its called when plugin is activated and visited the plugin page for first time
 * stores products in csv files in upload/feedo directory and send file_path to FEEDO server
 */ 
function feedo_product_upload( $per_page, $offset, $totalpage, $currentpage ) {
	ini_set('max_execution_time', 300);
	set_time_limit(0);
	$shopid = get_option('feedo_shopid');	
	$uploadpath = wp_upload_dir();
    $uploaddir = $uploadpath['basedir'];
    $upload_dir = $uploaddir . '/feedo';
    if ( ! is_dir($upload_dir) ) {
       mkdir( $upload_dir, 0755 );
    }	
	$currentDate = Inc\Common::currentDate();	
	$gproducts = Inc\Common::getProducts($per_page, $offset);
	$filename = $shopid.date('YmdHi').'_'.$currentpage.'.csv';
	$upload_dir = $uploadpath['basedir'].'/feedo/'.$filename;
	$upload_url = $uploadpath['baseurl'].'/feedo/'.$filename;
	try{
		$fp = fopen($upload_dir, 'w');
		if(!$fp){
			$tempdata = array('shopid' => $shopid,'status'=>"Unable to create file {$filename}",'step'=> 'CREATE FILE','date'=> '');
			$tempapi = Inc\Common::getServerData('api/update_track', $tempdata);
		}
	}catch(Exception $e){
		$m = "Error".$e->getMessage();
		$tempdata = array('shopid' => $shopid,'status'=> $m,'step'=> 'CREATE FILE','date'=> '');
		$tempapi = Inc\Common::getServerData('api/update_track', $tempdata);
	}
	$header = false;
	foreach( $gproducts as $prod ){
		if ( empty($header) ){
			$header = array_keys($prod);
			fputcsv($fp, $header);
			$header = array_flip($header);
		}
		fputcsv( $fp, array_merge($header, $prod) );			
	}
	fclose( $fp );
	$response = wp_remote_post( FEEDO_SERVER_URL.'api/productupload', array(
		'method' => 'POST',
		'timeout' => 200,
		'blocking' => true,
		'headers' => array(),
		'cookies' => array(),
		'body' => array('updatedDate'=> $currentDate,'shopid' =>$shopid,'fileurl'=>$upload_url,'totalfiles'=>$totalpage,'currentfile'=> $currentpage),
	));	
}
add_action( 'feedo_product_upload_hook','feedo_product_upload', 10, 4);


/**
 * Updates any product changes to FEEDO Server
 */
function feedo_product_update($per_page, $offset, $totalpage, $currentpage, $date) {
	ini_set('max_execution_time', 300);
	set_time_limit(0);	
	global $wpdb;
	$shopid = get_option('feedo_shopid');	
	$uploadpath   = wp_upload_dir();
	$currentDate = Inc\Common::currentDate();
	$parallel = $wpdb->get_results("SELECT count(ID) as total FROM {$wpdb->prefix}posts WHERE post_type='product' and post_status='publish' and post_modified >= '{$currentDate}'");
	$parallel = (int) $parallel[0]->total;
	if( $parallel > 0 ){
		$status = Inc\Common::getServerData('api/updateMiddleStatus?shopid='.$shopid.'&status=Y', array());
	}
	$gproducts = Inc\Common::getUpdatedProducts($date, $per_page, $offset);
	$filename = $shopid.date('YmdHi').'_'.$currentpage.'.csv';
	$upload_dir = $uploadpath['basedir'].'/feedo/'.$filename;
	$upload_url = $uploadpath['baseurl'].'/feedo/'.$filename;
	$fp = fopen($upload_dir, 'w');
	$header = false;
	foreach( $gproducts as $prod ){
		if (empty($header)){
			$header = array_keys($prod);
			fputcsv($fp, $header);
			$header = array_flip($header);
		}
		fputcsv( $fp, array_merge($header, $prod) );			
	}
	fclose( $fp );
	$response = wp_remote_post( FEEDO_SERVER_URL.'api/productupdate2', array(
		'method' => 'POST',
		'timeout' => 200,
		'blocking' => true,
		'headers' => array(),
		'cookies' => array(),
		'body' => array('updatedDate'=> $currentDate,'shopid' =>$shopid,'fileurl'=>$upload_url,'totalfiles'=>$totalpage,'currentfile'=> $currentpage),
	));	
	if($currentpage == $totalpage){
		$totalnew = $wpdb->get_results("SELECT count(ID) as total FROM {$wpdb->prefix}posts WHERE post_type='product' and post_status='publish' and post_modified >= '{$currentDate}'");
		$totalnew = (int) $totalnew[0]->total;
		if( $totalnew > 0 ){
			$status = Inc\Common::getServerData('api/updateMiddleStatus?shopid='.$shopid.'&status=Y', array());
		}else{
			$status = Inc\Common::getServerData('api/updateMiddleStatus?shopid='.$shopid.'&status=N', array());
		}
	}
}
add_action( 'feedo_product_update_hook','feedo_product_update', 10, 5);


/**
 * Initialize all the core classes of the plugin
 * uses composer to load all classes
 */
if ( class_exists( 'Inc\\Init' ) ) {
	Inc\Init::register_services();
}