<?php 
/**
 * @package  Feedo
 */
namespace Inc;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Common functions like method to call FEEDO server, get WooCommerce product
 */
class Common{
	
	// call to FEEDO server
	public static function getServerData($url,$data=NULL){		
		$ajaxurl = FEEDO_SERVER_URL .$url;
		$response = wp_remote_post( $ajaxurl, array(
			'method' => 'POST','timeout' => 45,'blocking' => true,'headers' => array(),'cookies' => array(),
			'body' => $data
		));		
		if ( is_wp_error( $response ) ) {
			return  array('ERROR'=> $response->get_error_message());   		
		}else{
			$apidata = wp_remote_retrieve_body( $response );			
			return json_decode($apidata);
		}
	}
	
	// return local datetime - used for sync-ing product changes
	public static function currentDate(){
		$currentDate = date_i18n('Y-m-d H:i:s');
		if(empty($currentDate)) $currentDate = current_time('mysql',0);
		return $currentDate;
	}

	// gets mapped google category for woocommerce category
	public static function getGoogleCat($wid,$shopid){
		return self::getServerData('api/get_google_category',array('id'=>$wid,'shopid'=>$shopid));
	}

	// gets google category name
	public static function getGoogleCatName($gcat){
		return self::getServerData('api/get_google_category_name',array('id'=>$gcat));
	}
	public static function getProductsByCat($name){
		$args = array('post_type' => 'product','post_status' => 'publish','posts_per_page' => -1,'product_cat' => "{$name}");
		$results = new \WP_Query($args);
		$posts = $results->posts;
		$p = array();
		foreach($posts as $post) {$p[] = $post->ID;}
		wp_reset_postdata();
		return $p;
	}

	// returns setting page tab structure
	public static function page_tabs( $current = 'first' ) {
	    $tabs = array(
	        'category'  => __( 'Mapping', 'plugin-textdomain' ),
	        'gmc'   => __( 'GMC', 'plugin-textdomain' )
	    );
	    $html = '<div class="sticky-header">';
	    foreach( $tabs as $tab => $name ){
	        $class = ( $tab == $current ) ? 'active' : '';
	        $html .= '<a class="' . $class . '" href="?page=feedo_settings&tab=' . $tab . '">' . $name . '</a>';
	    }
	    $html .= '</div>';
	    return $html;
	}

	// returns src url from img element 
	public static function getImageSrc($src){
		$pattern = '/src="([^"]*)"/';
		preg_match($pattern, $src, $matches);
		$return = $matches[1];
		unset($matches);
		return $return;
	}

	// returns all woocoomerce product in Google Product compatible format
	static function getProducts( $perpage, $offset ){
		global $wpdb;
		$results = $wpdb->get_results( "SELECT ID FROM {$wpdb->prefix}posts WHERE  post_type='product' AND post_status='publish' LIMIT {$offset}, {$perpage}", OBJECT );
		$gproducts = array();
		$wu = get_option('woocommerce_weight_unit');
		$du = get_option('woocommerce_dimension_unit');
		$shopid = get_option('feedo_shopid');
		foreach ($results as $prd) {
			$product = wc_get_product($prd->ID);
			$ptype = $product->get_type();
			if($ptype != 'external' && $ptype != 'grouped'){
				$data = array();			
				$data['id'] = $product->get_id();
				$data['product_id'] = $product->get_id();
				$data['title'] = $product->get_name();
				$data['sku'] = $product->get_sku();
				$description = $product->get_description();
				if($description) {
					$description = str_replace(array("\r", "\n", '"'), '', strip_tags($description));
					$description = preg_replace('#\[[^\]]+\]#', '',$description);
				}
				$data['description'] = $description;
				$data['link'] = $product->get_permalink();				
				$image = $product->get_image();
				if($image) $image= self::getImageSrc($image);
				$data['image_link'] = $image;
				$data['additional_image_link'] = '';
				$avail = $product->get_stock_status();
				if($avail=="instock") $avail = "in stock"; 
				elseif($avail=="outofstock") $avail = "out of stock";
				elseif($avail=="onbackorder") $avail = "preorder";
				$data['availability'] = $avail;
				$data['availability_date'] = '';				
				$price = $product->get_regular_price();
				if(empty($price)) $price = $product->get_price();				
				$price = str_replace(array(",", "'"), '', $price);
				$data['price'] = $price;
				$sale_price = $product->get_sale_price();
				$sale_price = str_replace(array(",", "'"), '', $sale_price);
				$data['sale_price'] = $sale_price;
				if($sale_price){
					$fromd = $product->get_date_on_sale_from();
						if($fromd) { $fromd = $fromd->getOffsetTimestamp();$fromd = date('Y-m-d', $fromd);}
					$tod = $product->get_date_on_sale_to();
						if($tod) { $tod = $tod->getOffsetTimestamp();$tod = date('Y-m-d', $tod);}					
					if($fromd && $tod){ 
						$data['sale_price_effective_date'] = $fromd."/".$tod;
					}
				}else{
					$data['sale_price_effective_date'] = "";
				}
				$data['unit_pricing_measure'] = "";
				$data['unit_pricing_base_measure'] = ""; 
				$data['installment'] = "";
				$data['loyalty_points'] = "";
				$data['excluded_destination'] = "";
				$data['adwords_labels'] = "";
				$data['adwords_grouping'] = "";		
				$data['google_product_category'] = "";
				$data['GMCTitle'] = "";
				$catids =  $product->get_category_ids();
				$cat = NULL;
				if($catids){
					$cat= self::getGoogleCat($catids[0],$shopid);
					if(!empty($cat)) $data['google_product_category'] = $cat->GPC_ID;
				}
				if(!empty($cat)){
					$GMCTitle= self::getGoogleCatName($cat->GPC_ID);
					if(!empty($GMCTitle)){ $data['GMCTitle'] = $GMCTitle->GMC_CAT_NAME;}
				}
				$data['product_type'] = "";
				$brand = $product->get_attribute('brand');
				if(empty($brand)) $brand = get_option('blogname');
				$data['brand'] = $brand;
				$data['gtin'] = $data['mpn'] = "";
				$gtin = $product->get_attribute('gtin');
				$mpn = $product->get_attribute('mpn');
				if(!empty($gtin)) $data['gtin'] = $gtin;
				if(!empty($mpn)) $data['mpn'] = $mpn;				
				if(empty($gtin) && empty($mpn)){
					$data['identifier_exists'] = "no";
				}
				$data['condition'] = "";
				$condition = $product->get_attribute('condition');
				if(!empty($condition)) $data['condition'] = $condition;
				$data['adult'] = "";
				$data['multipack'] = "";
				$data['is_bundle'] = "";
				$data['age_group'] = "";				
				$data['color'] ="";
				$color = $product->get_attribute('color');
				if(!empty($color)) $data['color'] = $color;
				$data['gender'] = "";
				$gender = $product->get_attribute('gender');
				if(!empty($gender)) $data['gender'] = $gender;
				$data['material'] = "";
				$material = $product->get_attribute('material');
				if(!empty($material)) $data['material'] = $material;
				$data['pattern'] = "";
				$pattern = $product->get_attribute('pattern');
				if(!empty($pattern)) $data['pattern'] = $pattern;				
				$data['size'] = "";
				$size = $product->get_attribute('size');
				if(!empty($size)) $data['size'] = $size;
				$data['status'] = "New";
				$data['size_type'] = "";
				$data['size_system'] = "";
				$data['item_group_id'] = "";
				$data['adwords_redirect'] = "";
				$data['custom_label_0'] = "";
				$data['custom_label_1'] = "";
				$data['custom_label_2'] = "";
				$data['custom_label_3'] = "";
				$data['custom_label_4'] = "";
				$data['shipping'] = "";
				$data['shipping_label'] = "";
				$data['shipping_weight'] = $data['shipping_length'] = $data['shipping_width'] = $data['shipping_height'] = '';
				$weight = $product->get_weight();
				if($weight) $data['shipping_weight'] = $weight.' '.$wu;
				$length = $product->get_length();
				if($length) $data['shipping_length'] = $length.' '.$du;
				$width = $product->get_width();
				if($width) $data['shipping_width'] = $width.' '.$du;
				$height = $product->get_height();
				if($height) $data['shipping_height'] = $height.' '.$du;
				$data['tax'] = "";
				$data['tax_category'] = "";
				$data['mobile_link'] = "";
				if($ptype == 'variable'){				
					$child_ids = $product->get_children();
					$data['child_ids'] = json_encode($child_ids);
					foreach($child_ids as $child_id){						
						$childProduct = wc_get_product($child_id);
						$data['id'] = $childProduct->get_id();
						$data['product_id'] = $childProduct->get_id();
						$data['item_group_id'] = $product->get_id();
						$data['title'] = $childProduct->get_name();
						$sku = $childProduct->get_sku();
						if($sku) $data['sku'] = $sku;
						$description = $childProduct->get_description();
						if($description){
							$description = str_replace(array("\r", "\n", '"'), '', strip_tags($description));
							$description = preg_replace('#\[[^\]]+\]#', '',$description);
							$data['description'] = $description;
						}				
						$data['link'] = $childProduct->get_permalink();
						$image = $childProduct->get_image();
						if($image){
							$image= self::getImageSrc($image);
							$data['image_link'] = $image;	
						}
						$avail = $childProduct->get_stock_status();
						if($avail=="instock") $avail = "in stock"; 
						elseif($avail=="outofstock") $avail = "out of stock";
						elseif($avail=="onbackorder") $avail = "preorder";					
						if($avail){
							$data['availability'] = $avail;
						}
						
						$price = $childProduct->get_regular_price();
						if(empty($price)) $price = $childProduct->get_price();						
						if($price) {
							$price = str_replace(array(",", "'"), '', $price);
							$data['price'] = $price;
						}						
						$sale_price = $childProduct->get_sale_price();
						if($sale_price) {
							$sale_price = str_replace(array(",", "'"), '', $sale_price);
							$data['sale_price'] = $sale_price;
						}
						if($sale_price){
							$fromd = $childProduct->get_date_on_sale_from();
								if($fromd) { $fromd = $fromd->getOffsetTimestamp();$fromd = date('Y-m-d', $fromd);}
							$tod = $childProduct->get_date_on_sale_to();
								if($tod) { $tod = $tod->getOffsetTimestamp();$tod = date('Y-m-d', $tod);}					
							if($fromd && $tod){ 
								$data['sale_price_effective_date'] = $fromd."/".$tod;
							}	
						}else{
							$data['sale_price_effective_date'] = "";
						}						
						$gtin = $childProduct->get_attribute('gtin');
						$mpn = $childProduct->get_attribute('mpn');
						if(!empty($gtin)) $data['gtin'] = $gtin;
						if(!empty($mpn)) $data['mpn'] = $mpn;
						if($gtin || $mpn) $data['identifier_exists'] = "yes";
						$condition = $childProduct->get_attribute('condition');
						if(!empty($condition)) $data['condition'] = $condition;
						$color = $childProduct->get_attribute('color');
						if(empty($color)) $color = $childProduct->get_attribute('colour');
						if(!empty($color)) $data['color'] = $color;
						$gender = $childProduct->get_attribute('gender');
						if(!empty($gender)) $data['gender'] = $gender;
						$material = $childProduct->get_attribute('material');
						if(!empty($material)) $data['material'] = $material;
						$pattern = $childProduct->get_attribute('pattern');
						if(!empty($pattern)) $data['pattern'] = $pattern;
						$size = $childProduct->get_attribute('size');
						if(!empty($size)) $data['size'] = $size;						
						$weight = $childProduct->get_weight();
						if($weight) $data['shipping_weight'] = $weight.' '.$wu;
						$length = $childProduct->get_length();
						if($length) $data['shipping_length'] = $length.' '.$du;
						$width = $childProduct->get_width();
						if($width) $data['shipping_width'] = $width.' '.$du;
						$height = $childProduct->get_height();
						if($height) $data['shipping_height'] = $height.' '.$du;
						array_push($gproducts, $data);
					}					
				}else{
					array_push($gproducts, $data);
				}
			}
		}
		return $gproducts;
	}
	
	// returns woocoomerce product in Google Product compatible format within time range
	static function getUpdatedProducts( $date, $perpage, $offset ){
		global $wpdb;
		$results = $wpdb->get_results("SELECT ID FROM {$wpdb->prefix}posts WHERE post_type='product' and post_status='publish' and post_modified >= '{$date}' LIMIT {$offset}, {$perpage}", OBJECT );
		$gproducts = array();
		$wu = get_option('woocommerce_weight_unit');
		$du = get_option('woocommerce_dimension_unit');
		$shopid = get_option('feedo_shopid');
		foreach ($results as $prd) {
			$product = wc_get_product($prd->ID);
			$ptype = $product->get_type();
			if($ptype != 'external' && $ptype != 'grouped'){
				$data = array();			
				$data['id'] = $product->get_id();
				$data['product_id'] = $product->get_id();
				$data['title'] = $product->get_name();
				$data['sku'] = $product->get_sku();
				$description = $product->get_description();
				if($description) {
					$description = str_replace(array("\r", "\n", '"'), '', strip_tags($description));
					$description = preg_replace('#\[[^\]]+\]#', '',$description);
				}
				$data['description'] = $description;
				$data['link'] = $product->get_permalink();				
				$image = $product->get_image();
				if($image) $image= self::getImageSrc($image);
				$data['image_link'] = $image;
				$data['additional_image_link'] = '';
				$avail = $product->get_stock_status();
				if($avail=="instock") $avail = "in stock"; 
				elseif($avail=="outofstock") $avail = "out of stock";
				elseif($avail=="onbackorder") $avail = "preorder";
				$data['availability'] = $avail;
				$data['availability_date'] = '';				
				$price = $product->get_regular_price();
				if(empty($price)) $price = $product->get_price();
				$price = str_replace(array(",", "'"), '', $price);
				$data['price'] = $price;
				$sale_price = $product->get_sale_price();
				$sale_price = str_replace(array(",", "'"), '', $sale_price);
				$data['sale_price'] = $sale_price;
				
				if($sale_price){
					$fromd = $product->get_date_on_sale_from();
						if($fromd) { $fromd = $fromd->getOffsetTimestamp();$fromd = date('Y-m-d', $fromd);}
					$tod = $product->get_date_on_sale_to();
						if($tod) { $tod = $tod->getOffsetTimestamp();$tod = date('Y-m-d', $tod);}
					if($fromd && $tod){ 
						$data['sale_price_effective_date'] = $fromd."/".$tod;
					}
				}else{
					$data['sale_price_effective_date'] = "";
				}
				$data['unit_pricing_measure'] = "";
				$data['unit_pricing_base_measure'] = ""; 
				$data['installment'] = "";
				$data['loyalty_points'] = "";
				$data['excluded_destination'] = "";
				$data['adwords_labels'] = "";
				$data['adwords_grouping'] = "";		
				$data['google_product_category'] = "";
				$data['GMCTitle'] = "";
				$catids =  $product->get_category_ids();
				$cat = NULL;
				if($catids){
					$cat= self::getGoogleCat($catids[0],$shopid);
					if(!empty($cat)) $data['google_product_category'] = $cat->GPC_ID;
				}
				if(!empty($cat)){
					$GMCTitle= self::getGoogleCatName($cat->GPC_ID);
					if(!empty($GMCTitle)){ $data['GMCTitle'] = $GMCTitle->GMC_CAT_NAME;}
				}
				$data['product_type'] = "";
				$brand = $product->get_attribute('brand');
				if(empty($brand)) $brand = get_option('blogname');
				$data['brand'] = $brand;
				$data['gtin'] = $data['mpn'] = "";
				$gtin = $product->get_attribute('gtin');
				$mpn = $product->get_attribute('mpn');
				if(!empty($gtin)) $data['gtin'] = $gtin;
				if(!empty($mpn)) $data['mpn'] = $mpn;				
				if(empty($gtin) && empty($mpn)){
					$data['identifier_exists'] = "no";
				}
				$data['condition'] = "";
				$condition = $product->get_attribute('condition');
				if(!empty($condition)) $data['condition'] = $condition;
				$data['adult'] = "";
				$data['multipack'] = "";
				$data['is_bundle'] = "";
				$data['age_group'] = "";				
				$data['color'] ="";
				$color = $product->get_attribute('color');
				if(!empty($color)) $data['color'] = $color;
				$data['gender'] = "";
				$gender = $product->get_attribute('gender');
				if(!empty($gender)) $data['gender'] = $gender;
				$data['material'] = "";
				$material = $product->get_attribute('material');
				if(!empty($material)) $data['material'] = $material;
				$data['pattern'] = "";
				$pattern = $product->get_attribute('pattern');
				if(!empty($pattern)) $data['pattern'] = $pattern;				
				$data['size'] = "";
				$size = $product->get_attribute('size');
				if(!empty($size)) $data['size'] = $size;
				$data['status'] = "New";
				$data['size_type'] = "";
				$data['size_system'] = "";
				$data['item_group_id'] = "";
				$data['adwords_redirect'] = "";
				$data['custom_label_0'] = "";
				$data['custom_label_1'] = "";
				$data['custom_label_2'] = "";
				$data['custom_label_3'] = "";
				$data['custom_label_4'] = "";
				$data['shipping'] = "";
				$data['shipping_label'] = "";
				$data['shipping_weight'] = $data['shipping_length'] = $data['shipping_width'] = $data['shipping_height'] = '';
				$weight = $product->get_weight();
				if($weight) $data['shipping_weight'] = $weight.' '.$wu;
				$length = $product->get_length();
				if($length) $data['shipping_length'] = $length.' '.$du;
				$width = $product->get_width();
				if($width) $data['shipping_width'] = $width.' '.$du;
				$height = $product->get_height();
				if($height) $data['shipping_height'] = $height.' '.$du;
				$data['tax'] = "";
				$data['tax_category'] = "";
				$data['mobile_link'] = "";
				if($ptype == 'variable'){				
					$child_ids = $product->get_children();
					$data['child_ids'] = json_encode($child_ids);
					foreach($child_ids as $child_id){						
						$childProduct = wc_get_product($child_id);
						$data['id'] = $childProduct->get_id();
						$data['product_id'] = $childProduct->get_id();
						$data['item_group_id'] = $product->get_id();
						$data['title'] = $childProduct->get_name();
						$sku = $childProduct->get_sku();
						if($sku) $data['sku'] = $sku;
						$description = $childProduct->get_description();
						if($description){
							$description = str_replace(array("\r", "\n", '"'), '', strip_tags($description));
							$description = preg_replace('#\[[^\]]+\]#', '',$description);
							$data['description'] = $description;
						}				
						$data['link'] = $childProduct->get_permalink();
						$image = $childProduct->get_image();
						if($image){
							$image= self::getImageSrc($image);
							$data['image_link'] = $image;	
						}
						$avail = $childProduct->get_stock_status();
						if($avail=="instock") $avail = "in stock"; 
						elseif($avail=="outofstock") $avail = "out of stock";
						elseif($avail=="onbackorder") $avail = "preorder";					
						if($avail){
							$data['availability'] = $avail;
						}
						$price = $childProduct->get_regular_price();
						if(empty($price)) $price = $childProduct->get_price();						
						if($price) {
							$price = str_replace(array(",", "'"), '', $price);
							$data['price'] = $price;
						}						
						$sale_price = $childProduct->get_sale_price();
						if($sale_price) {
							$sale_price = str_replace(array(",", "'"), '', $sale_price);
							$data['sale_price'] = $sale_price;
						}
						if($sale_price){
							$fromd = $childProduct->get_date_on_sale_from();
								if($fromd) { $fromd = $fromd->getOffsetTimestamp();$fromd = date('Y-m-d', $fromd);}
							$tod = $childProduct->get_date_on_sale_to();
								if($tod) { $tod = $tod->getOffsetTimestamp();$tod = date('Y-m-d', $tod);}					
							if($fromd && $tod){ 
								$data['sale_price_effective_date'] = $fromd."/".$tod;
							}
						}else{
							$data['sale_price_effective_date'] = "";
						}						
						$gtin = $childProduct->get_attribute('gtin');
						$mpn = $childProduct->get_attribute('mpn');
						if(!empty($gtin)) $data['gtin'] = $gtin;
						if(!empty($mpn)) $data['mpn'] = $mpn;
						if($gtin || $mpn) $data['identifier_exists'] = "yes";
						$condition = $childProduct->get_attribute('condition');
						if(!empty($condition)) $data['condition'] = $condition;
						$color = $childProduct->get_attribute('color');
						if(empty($color)) $color = $childProduct->get_attribute('colour');
						if(!empty($color)) $data['color'] = $color;
						$gender = $childProduct->get_attribute('gender');
						if(!empty($gender)) $data['gender'] = $gender;
						$material = $childProduct->get_attribute('material');
						if(!empty($material)) $data['material'] = $material;
						$pattern = $childProduct->get_attribute('pattern');
						if(!empty($pattern)) $data['pattern'] = $pattern;
						$size = $childProduct->get_attribute('size');
						if(!empty($size)) $data['size'] = $size;						
						$weight = $childProduct->get_weight();
						if($weight) $data['shipping_weight'] = $weight.' '.$wu;
						$length = $childProduct->get_length();
						if($length) $data['shipping_length'] = $length.' '.$du;
						$width = $childProduct->get_width();
						if($width) $data['shipping_width'] = $width.' '.$du;
						$height = $childProduct->get_height();
						if($height) $data['shipping_height'] = $height.' '.$du;
						array_push($gproducts, $data);
					}				
				}else{
					array_push($gproducts, $data);
				}
			}
		}
		return $gproducts;
	}
}