<?php 
/**
 * @package  Feedo
 */
 
namespace Inc\Pages;
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Main class for all actions
 * Adds admin menus
 * function for all pages 
 */
class Admin{
	
	public function register() {
		add_action( 'admin_menu', array( $this, 'feedo_add_admin_pages' ) );
		add_action( 'admin_menu', array( $this, 'feedo_add_admin_subpages' ) );
		add_action( 'admin_post_feedo_catmap_form', array( $this, 'feedo_update_catmap') );
		add_action( 'admin_post_feedo_catdel_form', array( $this, 'feedo_delete_catmap') );
	}
	
	// add menu link
	public function feedo_add_admin_pages() {
		add_menu_page( 'Feedo', 'Feedo', 'manage_options', 'feedo', array( $this, 'admin_index' ), FEEDO_PLUGIN_URL.'assets/images/feedo_dash.png', 58 );
	}
	
	// add sub menu links
	public function feedo_add_admin_subpages() {			
		add_submenu_page('feedo','Feedo','Feeds','manage_options','feedo',array( $this, 'admin_index'));
		add_submenu_page('feedo','Feedo Notification','Activity','manage_options','feedo_activity',array( $this, 'admin_activity'));	
		add_submenu_page('feedo','Feedo Settings','Settings','manage_options','feedo_settings',array( $this, 'admin_setting'));
		add_submenu_page(null,'Optimised Products','Optimised','manage_options','feedo_optimized',array( $this, 'admin_optimized'));	
		add_submenu_page(null,'Published Products','Published','manage_options','feedo_published',array( $this, 'admin_published'));
		add_submenu_page(null,'Edit Products','Edit','manage_options','feedo_edit_product',array( $this, 'edit_products'));
		add_submenu_page(null,'No Products','NP','manage_options','feedo_no_product',array( $this, 'no_products'));
	}
	
	// category page update function
	public function feedo_update_catmap(){
		if( isset( $_POST['action'] ) && isset( $_POST['woocats'] ) && isset( $_POST['gcats'] ) ) {		
			$shopid = get_option('feedo_shopid');
			$productarray = array();
			$wcat = sanitize_text_field($_POST['woocats']);			
			$gcat = intval($_POST['gcats']);
			$temp = explode("#",$wcat);
			$catname = $temp[1];
			$prds = \Inc\Common::getProductsByCat($catname);
			$options = array(
				'shopid' => $shopid,
				'wcat'=>$wcat,
				'gcat'=> $gcat,
				'products'=> $prds
			);
			$savecats = \Inc\Common::getServerData('api/save_category_map_new',$options);
			if ( ! session_id() ) {
				session_start();
			}
			$_SESSION['feedo_message'] = "Changes Saved Successfully.";
			wp_redirect(FEEDO_SETTINGURL);
			exit;
		}
	}
	public function feedo_delete_catmap(){
		if( isset( $_POST['action'] ) && isset( $_POST['catname'] ) && isset( $_POST['catid'] ) ) {	
			$shopid = get_option('feedo_shopid');
			$catname = sanitize_text_field($_POST['catname']);	
			$catid = intval($_POST['catid']);	
			$prds = \Inc\Common::getProductsByCat($catname);
			$options = array(
				'shopid' => $shopid,
				'id'=>$catid,
				'products'=> $prds
			);
			$savecats = \Inc\Common::getServerData('api/delete_category_mapping_new',$options);
			if ( ! session_id() ) {
				session_start();
			}
			$_SESSION['feedo_message'] = "Removed Successfully.";
			wp_redirect(FEEDO_SETTINGURL);
			exit;
		}
	}
	
	// loads main page and its functions
	public function admin_index() {
		// check if woocommerce is installed
		if ( ! class_exists( 'WooCommerce' ) ) {
			require_once FEEDO_PLUGIN_PATH . 'templates/woo.html';
			exit;
		}
		// checks for shopid
		$shopid = get_option('feedo_shopid');		
		if( !$shopid ){
			echo "Please Re Activate the Plugin..";
			exit;
		}
		// get status and details from FEEDO server
		$shopinfo = \Inc\Common::getServerData('api/get_info', array('shopid'=>$shopid));
		
		
		// ASK for user consent before anything 
		$IS_CONSENT = $shopinfo->IS_CONSENT;
		if( $IS_CONSENT != 'Y' ){ 
			require_once FEEDO_PLUGIN_PATH . 'templates/takeconsent.php';
			exit;			
		}
		
		// check if user logged in to GMC
		$IS_GMC_CONNECTED = $shopinfo->IS_GMC_CONNECTED;
		if( $IS_GMC_CONNECTED == 'N' ){
			require_once FEEDO_PLUGIN_PATH . 'templates/login.php';
			exit;
		}
		
		$merchantid = $shopinfo->MERCHANT_ID;		
		if( empty( $merchantid) ){
			require_once FEEDO_PLUGIN_PATH . 'templates/merchant.php';
			exit;
		}
		
		// checks if woocommerce products are uploaded to FEEDO
		$IS_UPDATED = $shopinfo->IS_UPDATED;
		$PROGRESS = $shopinfo->IS_UPDATE_IN_PROCESS;	
		if( $IS_UPDATED == 'N' ){
			// if uploading is in progress show loading page else upload products
			if( $PROGRESS == 'Y' ){
				$tempdata = array(
					'shopid' => $shopid,
					'status'=>json_encode($shopinfo),
					'step'=> 'ISUPDATED_Y',
					'date'=> ''
				);
				$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);				
				require_once FEEDO_PLUGIN_PATH . 'templates/firsttime.php';
				exit;
			}else{
				$tempdata = array(
					'shopid' => $shopid,
					'status'=>json_encode($shopinfo),
					'step'=> 'ISUPDATED_N',
					'date'=> ''
				);
				$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);
				$uploadpath = wp_upload_dir();
				$uploaddir = $uploadpath['basedir'];
				$upload_dir = $uploaddir . '/feedo';
				if ( ! is_dir($upload_dir) ) {
					if (!mkdir($upload_dir, 0755, true)) {
						$tempdata = array(
							'shopid' => $shopid,
							'status'=>json_encode($upload_dir),
							'step'=> 'MKDIR_ERROR',
							'date'=> ''
						);
						$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);
					}else{
						$tempdata = array(
							'shopid' => $shopid,
							'status'=>json_encode($upload_dir),
							'step'=> 'MKDIR_SUCCESS',
							'date'=> ''
						);
						$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);
					}				   
				}
				if ( ! is_dir($upload_dir) ) {
					// show permission page
					require_once FEEDO_PLUGIN_PATH . 'templates/filepermission.php';
					exit;
				}				
				
				global $wpdb;
				$per_page = 1000; // upload max 1000 products at a time
				$total = $wpdb->get_results("SELECT count(ID) as total FROM {$wpdb->prefix}posts WHERE  post_type='product' and post_status='publish'");	
				$total = (int) $total[0]->total;
				$tempdata = array(
					'shopid' => $shopid,
					'status'=>json_encode($total),
					'step'=> 'PROD_COUNT',
					'date'=> ''
				);
				$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);
				$setcount = \Inc\Common::getServerData('api/firstPercentage/'.$shopid, array());
				
				// if no products redirect them to woocommerce 
				if( $total < 1 ){
					require_once FEEDO_PLUGIN_PATH . 'templates/noproducts.php';
					exit;
				}					
				if( $total < 1000 ) $totalpage = 1;
				else {			
					$totalpage = (int) ( $total / $per_page );			
					$remainder = $total % $per_page;
					if( $remainder ) $totalpage = $totalpage +1;
				}
				// loop through 1000 products and trigger schedule single event to upload product				
				for( $currentpage=1; $currentpage <= $totalpage; $currentpage++ ){
					$offset = ( $currentpage * $per_page ) - $per_page;
					wp_schedule_single_event( time(), 'feedo_product_upload_hook', array( $per_page, $offset, $totalpage, $currentpage ) );	
				}		
				require_once FEEDO_PLUGIN_PATH . 'templates/firsttime.php';
				exit;
			}		
		}
		
		// check and sync if any product changes from last login
		$date = $shopinfo->LAST_UPDATED_DATE;		
		if( $date ){	
			global $wpdb;
			$total = $wpdb->get_results("SELECT count(ID) as total FROM {$wpdb->prefix}posts WHERE post_type='product' and post_status='publish' and post_modified >= '{$date}'");
			$total = (int) $total[0]->total;
			$tempdata = array(
					'shopid' => $shopid,
				'status'=>json_encode($total),
				'step'=> 'PROD_UPDATE_COUNT',
				'date'=> ''
			);
			$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);
				
			if( $total > 0 ){
				if( $PROGRESS == 'Y' ){
					require_once FEEDO_PLUGIN_PATH . 'templates/product_update.php';
					exit;
				}else{
					$status = \Inc\Common::getServerData('api/update_status', array('shopid'=>$shopid,'status'=>'Y'));
					
					$per_page = 500; // upload 500 product at a time	
					if( $total < 500 ) $totalpage = 1;
					else{
						$totalpage = (int) ( $total / $per_page );
						$remainder = $total % $per_page;
						if( $remainder ) $totalpage = $totalpage +1;	
					}
					$setcount = \Inc\Common::getServerData('api/firstPercentage/'.$shopid, array());
					for( $i=0; $i<$totalpage; $i++ ){
						$currentpage = $i+1;
						$offset = ( $currentpage * $per_page) - $per_page;
						wp_schedule_single_event( time(), 'feedo_product_update_hook', array( $per_page, $offset, $totalpage, $currentpage, $date ) );	
					}	
					require_once FEEDO_PLUGIN_PATH . 'templates/product_update.php';
					exit;
				}
			}
		}		
		require_once FEEDO_PLUGIN_PATH . 'templates/admin.php';			
	}

	// load activity page template - shows user actions
	public function admin_activity() {
		require_once FEEDO_PLUGIN_PATH . 'templates/notification.php';		
	}
	
	// load setting page - auto update & switch account
	public function admin_setting() {
		require_once FEEDO_PLUGIN_PATH . 'templates/setting.php';		
	}
	
	// load optimize page - lists optimized products
	public function admin_optimized() {
		$shopid = get_option('feedo_shopid');
		require_once FEEDO_PLUGIN_PATH . 'templates/optimised.php';	
	}

	// loads published page - lists products from Google Merchant Center products of user
	public function admin_published() {
		$shopid = get_option('feedo_shopid');
		require_once FEEDO_PLUGIN_PATH . 'templates/published.php';	
	}

	// update product edits to FEEDO (will not modifiy woocommerce products)
	public function edit_products(){
		if (isset( $_GET['rowId'] ) && isset( $_GET['feedId'] )) {
			$feedId = $_GET['feedId'];
			$rowId = $_GET['rowId'];
			$shopId = get_option('feedo_shopid');			
			$response = wp_remote_post( FEEDO_SERVER_URL.'api/get_feed_product', array(
				'method' => 'POST',
				'timeout' => 200,
				'blocking' => true,
				'headers' => array(),
				'cookies' => array(),
				'body' => array('feedId'=> $feedId,'rowId' =>$rowId,'shopId'=>$shopId),
			));
			$tempdata = array(
				'shopid' => $shopId,
				'status'=>$rowId.':'.$feedId,
				'step'=> 'EDIT_SUBMIT',
				'date'=> ''
			);
			$tempapi = \Inc\Common::getServerData('api/update_track', $tempdata);

			$productinfo = wp_remote_retrieve_body( $response );
			$productinfo = json_decode($productinfo);
			require_once FEEDO_PLUGIN_PATH . 'templates/editProduct.php';
			exit;
		}else{
			exit( wp_safe_redirect(FEEDO_PLUGINURL) );
		}
	}
	
	// show no-product template when no products in WooCommerce
	public function no_products() {
		require_once FEEDO_PLUGIN_PATH . 'templates/noproducts.php';		
	}
}